@@ -120,7 +120,9 @@ iStringList *   split_String        (const iString *, const char *separator);
 iString *       urlEncode_String    (const iString *);
 iString *       urlEncodeExclude_String(const iString *d, const char *excluded);
 iString *       urlDecode_String    (const iString *);
+iString *       urlDecodeExclude_String(const iString *, const char *excluded);
 iChar           first_String        (const iString *);
+iChar           last_String         (const iString *);
 iBlock *        toLocal_String      (const iString *);
 iBlock *        toUtf16_String      (const iString *);
 
@@ -257,6 +259,14 @@ iLocalDef iBool equal_Rangecc(const iRangecc d, const char *cstr) {
 iLocalDef iBool equalCase_Rangecc(const iRangecc d, const char *cstr) {
     return cmpCStrSc_Rangecc(d, cstr, &iCaseInsensitive) == 0;
 }
+iLocalDef iBool equalRange_Rangecc(const iRangecc d, const iRangecc other) {
+    return size_Range(&d) == size_Range(&other) &&
+           cmpCStrNSc_Rangecc(d, other.start, size_Range(&d), &iCaseSensitive) == 0;
+}
+iLocalDef iBool equalRangeCase_Rangecc(const iRangecc d, const iRangecc other) {
+    return size_Range(&d) == size_Range(&other) &&
+           cmpCStrNSc_Rangecc(d, other.start, size_Range(&d), &iCaseInsensitive) == 0;
+}
 iLocalDef iBool startsWith_Rangecc(const iRangecc d, const char *cstr) {
     return startsWithSc_Rangecc(d, cstr, &iCaseSensitive);
 }
