@@ -258,6 +258,7 @@ static void init_Socket_(iSocket *d) {
     d->error = NULL;
     d->readyRead = NULL;
     d->writeFinished = NULL;
+    d->status = initialized_SocketStatus;
 }
 
 void deinit_Socket(iSocket *d) {
@@ -265,8 +266,9 @@ void deinit_Socket(iSocket *d) {
     iGuardMutex(&d->mutex, {
         iReleasePtr(&d->output);
         iReleasePtr(&d->input);
-        iReleasePtr(&d->address);
     });
+    waitForFinished_Address(d->address);
+    iReleasePtr(&d->address);
     deinit_Mutex(&d->mutex);
     delete_Pipe(d->stopConnect);
     deinit_Condition(&d->allSent);
@@ -406,8 +408,6 @@ static iThreadResult connectAsync_Socket_(iThread *thd) {
                 }
             }
             lock_Mutex(&d->mutex);
-            delete_Pipe(d->stopConnect);
-            d->stopConnect = NULL;
             if (d->status == connecting_SocketStatus) {
                 if (rc == 0) {
                     setStatus_Socket_(d, connected_SocketStatus);
@@ -422,6 +422,8 @@ static iThreadResult connectAsync_Socket_(iThread *thd) {
             unlock_Mutex(&d->mutex);
         }
     }
+    delete_Pipe(d->stopConnect);
+    d->stopConnect = NULL;
     if (rc) {
         int errNum;
         char *msg;
