@@ -51,6 +51,7 @@ static iBool isPrngSeeded_;
 
 struct Impl_Context {
     SSL_CTX *ctx;
+    X509_STORE *certStore;
 };
 
 void init_Context(iContext *d) {
@@ -73,6 +74,7 @@ void init_Context(iContext *d) {
     SSL_CTX_set_verify(d->ctx, SSL_VERIFY_NONE, NULL); /* allow manual verification */
     /* Bug workarounds: https://www.openssl.org/docs/manmaster/man3/SSL_CTX_set_options.html */
     SSL_CTX_set_options(d->ctx, SSL_OP_ALL);
+    d->certStore = NULL;
 }
 
 void deinit_Context(iContext *d) {
@@ -83,6 +85,19 @@ iBool isValid_Context(iContext *d) {
     return d->ctx != NULL;
 }
 
+void setCACertificates_TlsRequest(const iString *caFile, const iString *caPath) {
+    iContext *d = context_;
+    if (d->certStore) {
+        X509_STORE_free(d->certStore);
+    }
+    d->certStore = X509_STORE_new();
+    if (!X509_STORE_load_locations(d->certStore,
+                                   isEmpty_String(caFile) ? NULL : cstr_String(caFile),
+                                   isEmpty_String(caPath) ? NULL : cstr_String(caPath))) {
+        iWarning("[TlsRequest] OpenSSL failed to load CA certificates\n");
+    }
+}
+
 iDefineTypeConstruction(Context)
 
 static iBool readAllFromBIO_(BIO *bio, iBlock *out) {
@@ -104,28 +119,40 @@ static iBool readAllFromBIO_(BIO *bio, iBlock *out) {
 
 struct Impl_TlsCertificate {
     X509 *cert;
+    STACK_OF(X509) *chain;
     EVP_PKEY *pkey;
 };
 
 iDefineTypeConstruction(TlsCertificate)
 
 void init_TlsCertificate(iTlsCertificate *d) {
-    d->cert = NULL;
-    d->pkey = NULL;
+    d->cert  = NULL;
+    d->chain = NULL;
+    d->pkey  = NULL;
 }
 
 void deinit_TlsCertificate(iTlsCertificate *d) {
     if (d->cert) {
         X509_free(d->cert);
     }
+    if (d->chain) {
+        sk_X509_free(d->chain);
+    }
     if (d->pkey) {
         EVP_PKEY_free(d->pkey);
     }
 }
 
-static iTlsCertificate *newX509_TlsCertificate_(X509 *cert) {
+//static iTlsCertificate *newX509_TlsCertificate_(X509 *cert) {
+//    iTlsCertificate *d = new_TlsCertificate();
+//    d->cert = cert;
+//    return d;
+//}
+
+static iTlsCertificate *newX509Chain_TlsCertificate_(X509 *cert, STACK_OF(X509) *chain) {
     iTlsCertificate *d = new_TlsCertificate();
-    d->cert = cert;
+    d->cert  = cert;
+    d->chain = chain;
     return d;
 }
 
@@ -296,6 +323,29 @@ iBool isExpired_TlsCertificate(const iTlsCertificate *d) {
     return X509_cmp_current_time(X509_get0_notAfter(d->cert)) < 0;
 }
 
+enum iTlsCertificateVerifyStatus verify_TlsCertificate(const iTlsCertificate *d) {
+    enum iTlsCertificateVerifyStatus status = unverified_TlsCertificateVerifyStatus;
+    if (!d->cert) {
+        return status;
+    }
+    X509_STORE_CTX *store = X509_STORE_CTX_new();
+    X509_STORE_CTX_init(store, context_->certStore, d->cert, d->chain);
+    const int result = X509_verify_cert(store);
+    const int err    = X509_STORE_CTX_get_error(store);
+    if (err == X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT) {
+        status = selfSigned_TlsCertificateVerifyStatus;
+    }
+    else if (result) {
+        status = authority_TlsCertificateVerifyStatus;
+    }
+    iDebug("[TlsCertificate] result:%d %s (error:%d)\n",
+           result,
+           X509_verify_cert_error_string(err),
+           err);
+    X509_STORE_CTX_free(store);
+    return status;
+}
+
 iBool verifyDomain_TlsCertificate(const iTlsCertificate *d, iRangecc domain) {
     if (!d->cert) return iFalse;
     /* Check the common name first manually. X509_check_host() seems to prioritize SAN,
@@ -456,13 +506,20 @@ static enum iSSLResult sslResult_TlsRequest_(iTlsRequest *d, int code) {
 }
 
 static void setStatus_TlsRequest_(iTlsRequest *d, enum iTlsRequestStatus st) {
-    iGuardMutex(&d->mtx, if (d->status != st) {
+    lock_Mutex(&d->mtx);
+    if (d->status != st) {
         d->status = st;
-        signal_Condition(&d->gotIncoming); /* wake up if sleeping */
         if (st == finished_TlsRequestStatus || st == error_TlsRequestStatus) {
             signalAll_Condition(&d->requestDone);
         }
-    });
+        unlock_Mutex(&d->mtx);
+        lock_Mutex(&d->incomingMtx);
+        signal_Condition(&d->gotIncoming); /* wake up if sleeping */
+        unlock_Mutex(&d->incomingMtx);
+    }
+    else {
+        unlock_Mutex(&d->mtx);
+    }
 }
 
 static void flushToSocket_TlsRequest_(iTlsRequest *d) {
@@ -513,11 +570,20 @@ static iBool encrypt_TlsRequest_(iTlsRequest *d) {
     return iTrue;
 }
 
-void init_TlsRequest(iTlsRequest *d) {
+static void initContext_(void) {
     if (!context_) {
         context_ = new_Context();
         atexit(globalCleanup_TlsRequest_);
     }
+}
+
+void setCiphers_TlsRequest(const char *cipherList) {
+    initContext_();
+    SSL_CTX_set_cipher_list(context_->ctx, cipherList);
+}
+
+void init_TlsRequest(iTlsRequest *d) {
+    initContext_();
     init_Mutex(&d->mtx);
     d->hostName = new_String();
     d->port = 0;
@@ -548,8 +614,8 @@ void init_TlsRequest(iTlsRequest *d) {
 }
 
 void deinit_TlsRequest(iTlsRequest *d) {
-    signal_Condition(&d->gotIncoming);
-    d->status = finished_TlsRequestStatus;
+    iGuardMutex(&d->incomingMtx, signal_Condition(&d->gotIncoming));
+    iGuardMutex(&d->mtx, d->status = finished_TlsRequestStatus);
     if (d->thread) {
         join_Thread(d->thread);
         iRelease(d->thread);
@@ -618,7 +684,9 @@ static int processIncoming_TlsRequest_(iTlsRequest *d, const char *src, size_t l
             }
         }
         if (!d->cert) {
-            d->cert = newX509_TlsCertificate_(SSL_get_peer_certificate(d->ssl));
+            const STACK_OF(X509) *chain = SSL_get_peer_cert_chain(d->ssl);
+            d->cert = newX509Chain_TlsCertificate_(SSL_get_peer_certificate(d->ssl),
+                                                   sk_X509_dup(chain));
         }
         /* The encrypted data is now in the input bio so now we can perform actual
            read of unencrypted data. */
@@ -675,6 +743,8 @@ static iBool readIncoming_TlsRequest_(iTlsRequest *d) {
 }
 
 static iThreadResult run_TlsRequest_(iThread *thread) {
+    /* TODO: This thread seems unnecessary. The Socket already runs a thread -- we could execute
+       this via its observers, and block to encrypt outgoing data. */
     iTlsRequest *d = userData_Thread(thread);
     doHandshake_TlsRequest_(d);
     for (;;) {
@@ -682,14 +752,17 @@ static iThreadResult run_TlsRequest_(iThread *thread) {
         if (!readIncoming_TlsRequest_(d)) {
             lock_Mutex(&d->mtx);
             if (d->status == submitted_TlsRequestStatus) {
-                wait_Condition(&d->gotIncoming, &d->mtx);
+                unlock_Mutex(&d->mtx);
+                /* Wait for incoming data. */
+                lock_Mutex(&d->incomingMtx);
+                wait_Condition(&d->gotIncoming, &d->incomingMtx);
+                unlock_Mutex(&d->incomingMtx);
             }
             else {
 //                fprintf(stderr, "[TlsRequest] run loop exiting, status %d\n", d->status);
                 unlock_Mutex(&d->mtx);
                 break;
             }
-            unlock_Mutex(&d->mtx);
         }
     }
     readIncoming_TlsRequest_(d);
@@ -762,6 +835,9 @@ void cancel_TlsRequest(iTlsRequest *d) {
         unlock_Mutex(&d->mtx);
         close_Socket(d->socket);
     }
+    else {
+        unlock_Mutex(&d->mtx);
+    }
     signal_Condition(&d->gotIncoming);
     join_Thread(d->thread);
 }
