@@ -133,6 +133,7 @@ struct Impl_GmRequest {
     iTlsRequest *        req;
     iGopher              gopher;
     iGmResponse *        resp;
+    iBool                isFilterEnabled;
     iBool                isRespLocked;
     iBool                isRespFiltered;
     iAtomicInt           allowUpdate;
@@ -208,7 +209,7 @@ static int processIncomingData_GmRequest_(iGmRequest *d, const iBlock *data) {
                 resp->statusCode = code;
                 d->state         = receivingBody_GmRequestState;
                 notifyUpdate     = iTrue;
-                if (willTryFilter_MimeHooks(mimeHooks_App(), &resp->meta)) {
+                if (d->isFilterEnabled && willTryFilter_MimeHooks(mimeHooks_App(), &resp->meta)) {
                     d->isRespFiltered = iTrue;
                 }
             }
@@ -226,7 +227,12 @@ static int processIncomingData_GmRequest_(iGmRequest *d, const iBlock *data) {
 static void readIncoming_GmRequest_(iGmRequest *d, iTlsRequest *req) {
     lock_Mutex(d->mtx);
     iGmResponse *resp = d->resp;
-    iAssert(d->state != finished_GmRequestState); /* notifications out of order? */
+    if (d->state == finished_GmRequestState || d->state == failure_GmRequestState) {
+        /* The request has already finished or been aborted (e.g., invalid header). */
+        delete_Block(readAll_TlsRequest(req));
+        unlock_Mutex(d->mtx);
+        return;
+    }
     iBlock *  data         = readAll_TlsRequest(req);
     const int ubits        = processIncomingData_GmRequest_(d, data);
     iBool     notifyUpdate = (ubits & 1) != 0;
@@ -457,8 +463,9 @@ static void beginGopherConnection_GmRequest_(iGmRequest *d, const iString *host,
 void init_GmRequest(iGmRequest *d, iGmCerts *certs) {
     d->mtx = new_Mutex();
     d->resp = new_GmResponse();
-    d->isRespLocked = iFalse;
-    d->isRespFiltered = iFalse;
+    d->isFilterEnabled = iTrue;
+    d->isRespLocked    = iFalse;
+    d->isRespFiltered  = iFalse;
     set_Atomic(&d->allowUpdate, iTrue);
     init_String(&d->url);
     init_Gopher(&d->gopher);
@@ -492,6 +499,10 @@ void deinit_GmRequest(iGmRequest *d) {
     delete_Mutex(d->mtx);
 }
 
+void enableFilters_GmRequest(iGmRequest *d, iBool enable) {
+    d->isFilterEnabled = enable;
+}
+
 void setUrl_GmRequest(iGmRequest *d, const iString *url) {
     set_String(&d->url, urlFragmentStripped_String(url));
     /* Encode hostname to Punycode here because we want to submit the Punycode domain name
@@ -546,7 +557,7 @@ void submit_GmRequest(iGmRequest *d) {
             remove_Block(&path->chars, 0, 1);
         }
 #endif
-        iFile *  f    = new_File(path);
+        iFile *f = new_File(path);
         if (open_File(f, readOnly_FileMode)) {
             /* TODO: Check supported file types: images, audio */
             /* TODO: Detect text files based on contents? E.g., is the content valid UTF-8. */
