@@ -75,7 +75,8 @@ enum iAnimFlag {
     indefinite_AnimFlag = iBit(1), /* does not end; must be linear */
     easeIn_AnimFlag     = iBit(2),
     easeOut_AnimFlag    = iBit(3),
-    easeBoth_AnimFlag = easeIn_AnimFlag | easeOut_AnimFlag,
+    easeBoth_AnimFlag   = easeIn_AnimFlag | easeOut_AnimFlag,
+    softer_AnimFlag     = iBit(4),
 };
 
 struct Impl_Anim {
@@ -198,7 +199,7 @@ iWidget *   makeValueInput_Widget   (iWidget *parent, const iString *initialValu
 void        updateValueInput_Widget (iWidget *, const char *title, const char *prompt);
 iWidget *   makeMessage_Widget      (const char *title, const char *msg);
 iWidget *   makeQuestion_Widget     (const char *title, const char *msg,
-                                     const char *labels[], const char *commands[], size_t count);
+                                     const iMenuItem *items, size_t numItems);
 
 iWidget *   makePreferences_Widget      (void);
 iWidget *   makeBookmarkEditor_Widget   (void);
